#include <vector>


#include "Colorization.h"
#include "calcSpinCurvC.h"
#include "GetColorExact.h"
#include <vcg/complex/append.h>



// input params
bool const USE_FIELD = false;      // use/don't use field
//const char *filename = "sappho_input_full.ply";
//const char *filename = "octopus.ply";
//const char *filename = "torch.ply";
const Color4b EmptyColor = Color4b (150,150,150,255);
const Color4b SepcialColor = Color4b (1,1,1,255);
const int SPIN_IMAGE_SIZE = 8;


vector<int> getColoredVertices (MyMesh *mesh)
{
	vector <int>  colorVerInds;
	for (int i= 0 ; i < mesh ->vn; i++)
	{
		if (mesh -> vert[i].cC() != EmptyColor)
		{
			colorVerInds.push_back(i);
		}
	}
	return colorVerInds;
}

vector<int> getSpecialColoredVertices (MyMesh *mesh)
{
	vector <int>  specialColorVerInds;
	for (int i= 0 ; i < mesh ->vn; i++)
	{
		if (mesh -> vert[i].cC() == SepcialColor)
		{
			specialColorVerInds.push_back(i);
		}
	}
	return specialColorVerInds;
}

void colorMesh (MyMesh *inMesh, MyMesh *outMesh, bool use_field)
{
	// calc descriptor
	vector <double *> descriptor = calcSpinCurv (inMesh, SPIN_IMAGE_SIZE);
	// calc field
	if (use_field)
	{
		printf ("use field is not supported yet !!!\n");
	}


	// find the colored vertices
	vector <int> colorVerInds = getColoredVertices(inMesh);
	getColorExact(inMesh, descriptor, SPIN_IMAGE_SIZE, colorVerInds, outMesh);
}

int main(int argc,char ** argv)
{
	if(argc != 2) 
	{
		printf("Usage: Colorization <filename>\n");
		return 0;
	}
	const char *filename = argv[1];

	MyMesh inMesh;
	MyMesh *outMesh = new MyMesh ();

	//open a mesh
	int err = tri::io::Importer<MyMesh>::Open(inMesh,filename);
	if(err) { // all the importers return 0 in case of success
      printf("Error in reading %s: '%s'\n",filename, tri::io::Importer<MyMesh>::ErrorMsg(err));
      exit(-1);
    }

	vcg::tri::Append <MyMesh,MyMesh>::MeshCopy (*outMesh, inMesh);
	colorMesh (&inMesh, outMesh, USE_FIELD);


	tri::io::ExporterPLY<MyMesh>::Save(*outMesh,"out.ply",tri::io::Mask::IOM_VERTCOLOR);
	delete outMesh;
	return 0;
}

