#include "Colorization.h"
#include "GetColorExact.h"
#include "calcDifDist.h"

#include <vcg/complex/algorithms/update/normal.h>
#include <vcg/complex/algorithms/update/topology.h>
#include <vcg/complex/algorithms/update/curvature.h>
#include <vcg/complex/append.h>
#include <vcg/simplex/edge/base.h>
#include <vcg/simplex/face/topology.h>
#include <vcg/simplex/face/pos.h>     

#include <Eigen/Sparse>
#include <vector>
#include <Eigen/IterativeLinearSolvers>



//function that finds all the neigbours of the vertex (ind) using the adjecent matrix in mesh
void findNeigb(int ind, MyMesh *mesh, vector<int> &neigbours)
{
	
	MyVertex *v = &(mesh->vert[ind]), *tempV;
	if ((*v).VFp() != NULL) {


	MyFace* start = (*v).VFp();
	vcg::face::JumpingPos<MyFace> p(start,v);// constructor that takes face, edge and vertex
	do
	{
		p.NextFE();
		//p.FlipF();		p.FlipE();
		MyFace* f = p.F();
		for (int i=0;i<3;i++) {
			int ind1 = f->V(i)->Flags();
			if (ind != ind1)
				neigbours.push_back(ind1);
		}

	}while(p.f!=start);


	}
	
	sort(neigbours.begin(), neigbours.end());
	neigbours.erase(unique(neigbours.begin(), neigbours.end()), neigbours.end()); 
}
	

inline Color4d rgb2ntsc (Color4b rgbColor)
{
	return Color4d(
		(0.299*rgbColor.X()+0.587*rgbColor.Y()+0.114*rgbColor.Z())/256,
		(0.596*rgbColor.X()-0.274*rgbColor.Y()-0.322*rgbColor.Z())/256,
		(0.211*rgbColor.X()-0.523*rgbColor.Y()+0.312*rgbColor.Z())/256,
		1);
}

inline Color4b ntsc2rgb (double Y, double I, double Q)
{
	return Color4b(
		(1*Y+0.956*I+0.621*Q)*256,
		(1*Y-0.272*I-0.647*Q)*256,
		(1*Y-1.106*I+1.703*Q)*256,
		256);
}


void getColorExact (MyMesh *mesh, vector <double *> &descriptor, int spinImageSize, vector <int> &colorVerInds, MyMesh *outMesh)
{

	int vertexCount = mesh -> vn;
	int estimation_of_entries = vertexCount*5;

	typedef Eigen::SparseMatrix<double> SpMat;

	SpMat A(vertexCount,vertexCount);

	//lblIndsBool (Vector of 0/1 ) where vertex is coloroed or not. length is number of verteces
	bool  *lblIndsBool = new bool[vertexCount];
	vector <int>::iterator It;
	vector <double>::iterator it_double;
	int index;

	memset (lblIndsBool, false, vertexCount);
	for (It = colorVerInds.begin(); It != colorVerInds.end(); ++It)
	{
		index = *It;
		lblIndsBool[index] = true;
	}

	vector <int> row_inds;
    vector <int> col_inds;
	vector <double> vals;
	row_inds.reserve(estimation_of_entries);
	col_inds.reserve(estimation_of_entries);
	vals.reserve(estimation_of_entries);

	//TODO: put in flags vertex index for neighbour retrievals
	for (int i= 0 ; i < mesh ->vn; i++)
	{
		mesh -> vert[i].SetFlags(i);
	}


	for (int ind=0; ind < vertexCount; ind++)
	{
      if (!lblIndsBool[ind])
	  {
        vector <int> neighbors;
		findNeigb(ind, mesh, neighbors);
		for (It = neighbors.begin(); It != neighbors.end(); ++It)
		{
			int neigbInd = *It;
			row_inds.push_back(ind);
            col_inds.push_back(neigbInd);
		}

        vector <double> dists;
		descriptor_dists(descriptor, spinImageSize, neighbors, ind, dists);
     
		double c_var = 0.0;
		for (it_double = dists.begin(); it_double != dists.end(); ++it_double){
			double dist = *it_double;
			c_var += dist * dist;
		}
		c_var /= dists.size();
        double csig = c_var*0.15;

        if (csig < 0.00001)
           csig = 0.00001;

		vector <double> costs;
		for (it_double = dists.begin(); it_double != dists.end(); ++it_double){
			double dist = *it_double;
			costs.push_back(exp(-1.0 * (dist * dist) / csig)); 
		}
    
        double sum_of_costs = 0.0;
		for (it_double = costs.begin(); it_double != costs.end(); ++it_double){
			double cost = *it_double;
			sum_of_costs += cost;
		}

		for (it_double = costs.begin(); it_double != costs.end(); ++it_double){
			double cost = *it_double;
			vals.push_back(-1.0 * cost / sum_of_costs); 
		}

	  }
      row_inds.push_back(ind);
      col_inds.push_back(ind);
      vals.push_back(1.0); 

	}        

	//create sparse matrix
    typedef Eigen::Triplet<double> T;
    std::vector<T> tripletList;
    tripletList.reserve(estimation_of_entries);
	vector <int>::iterator rIt = row_inds.begin();
	vector <int>::iterator cIt = col_inds.begin();
	vector <double>::iterator vIt = vals.begin();

	printf ("\n \n");

	for (; rIt != row_inds.end(); ++rIt,  ++cIt, ++vIt)
	{
		tripletList.push_back(T(*rIt, *cIt, *vIt));
		//tripletList.push_back(T(*cIt, *rIt, *vIt));
		//printf ("%5d %5d %8.3f; ... \n", *rIt, *cIt, *vIt);
    }
	//printf ("\n \n");

    A.setFromTriplets(tripletList.begin(), tripletList.end());
	A.finalize();
    // A is ready to go!  


	// A = sparse matrix. size is vertices * vertices.  val is the true issue.
	// b = double vector. length is number of verteces. 0 - no color, else ntsc (1 to 3)
	// new_val = A\b;
	// color = new_val ntsc to rgb + small calc.

	Eigen::VectorXd bY(vertexCount), bI(vertexCount), bQ(vertexCount);
	for (int i=0 ; i < vertexCount ; i++)
	{
		bY(i) = 0; 
		bI(i) = 0; 
		bQ(i) = 0; 
	}

	Color4d ntscColor;
	for (It = colorVerInds.begin(); It != colorVerInds.end(); ++It)
	{
		index = *It;
		ntscColor = rgb2ntsc (mesh -> vert[index].cC());
		bY(index) = ntscColor.X(); 
		bI(index) = ntscColor.Y(); 
		bQ(index) = ntscColor.Z(); 
	}

	
	Eigen::VectorXd newY(vertexCount), newI(vertexCount), newQ(vertexCount);

		
	 // Solving:
	Eigen::BiCGSTAB<SpMat, Eigen::IncompleteLUT<double>> solver(A);
	//solver.analyzePattern(A);
	//solver.factorize(A);
	//solver.compute(A);
	
	//solver.setTolerance(solver.tolerance() / 10.0);
	solver.setMaxIterations(solver.maxIterations()*2);

	newY = solver.solve(bY);
	if(solver.info()!=Eigen::Success)
		printf ("Warning:  decomposition failed: %d\n", solver.info());
	newI = solver.solve(bI);
	if(solver.info()!=Eigen::Success)
		printf ("Warning:  decomposition failed: %d\n", solver.info());
	newQ = solver.solve(bQ);
	if(solver.info()!=Eigen::Success)
		printf ("Warning:  decomposition failed: %d\n", solver.info());


	/*
	printf ("\n \n");
	for (int i=0 ; i < vertexCount ; i++)
	{
		printf ("%8.3f %8.3f; ... \n", bY(i), newY(i));
    }
	printf ("\n \n");
	*/

	for (int i=0 ; i < vertexCount ; i++)
	{
		outMesh -> vert[i].C() = ntsc2rgb (newY(i),newI(i),newQ(i));
	}
	delete [] lblIndsBool;
}

