#include "Colorization.h"
#include "calcDifDist.h"

#include "dd_head.h"
//#include <stdio.h>
//#include <stdlib.h>
//#include "math.h"

///////////////////////////////////////////////////////////////////////////////////////////
// Construction, deconstruction 
///////////////////////////////////////////////////////////////////////////////////////////
dd_dist::dd_dist(int n1, int n2)
{
	m_nSize[0]=n1; m_nSize[1]=n2;

    m_dDiff2D = new double*[n1];
    m_dTmp2D  = new double*[n1];
    for(int y=0; y<n1; ++y)	{
        m_dDiff2D[y] = new double[n2];
        m_dTmp2D[y] = new double[n2];
    }
}

dd_dist::~dd_dist()
{
	int n1 = m_nSize[0];
	for(int y=0; y<n1; ++y)	{
        delete[] m_dDiff2D[y];
        delete[] m_dTmp2D[y];
    }
    delete[] m_dDiff2D;
    delete[] m_dTmp2D;
}
///////////////////////////////////////////////////////////////////////////////////////////
//			Diffusion-based distance
///////////////////////////////////////////////////////////////////////////////////////////

// diffision-based distance between two 2d hstograms of size n1 X n2
double dd_dist::dd2D(double* h1, double* h2)
{
	int n1 = m_nSize[0];
    int n2 = m_nSize[1];

	//- compute the difference 
	double	*p1	= h1;
	double	*p2	= h2;
	int		x,y;
	for(y=0; y<n1; ++y)	
		for(x=0; x<n2; ++x)	
			m_dDiff2D[y][x]	= *(p1++)-*(p2++);

	//- compute the distance recursively
	double	dDist	= L1_dist(n1,n2);
	while(n1>1 && n2>1)	{
		SmoothDiff2D(n1,n2);			// smooth
		DownsampleDiff2D(n1,n2);		// downsampling
		dDist	+= L1_dist(n1,n2);		// distance
	}
	return dDist;
}

////////////////////////////////////////////////////////////////////////////////////////////
void dd_dist::DownsampleDiff2D(int& n1, int &n2)
{
	int		x,y;
	n1	= n1/2;
	for(y=0;y<n1;++y) 
	for(x=0;x<n2;++x)
		m_dDiff2D[y][x]	= m_dDiff2D[2*y][x]+m_dDiff2D[2*y+1][x];

	n2	= n2/2;
	for(y=0;y<n1;++y) 
	for(x=0;x<n2;++x)
		m_dDiff2D[y][x]	= m_dDiff2D[y][2*x]+m_dDiff2D[y][2*x+1];
}


void dd_dist::SmoothDiff2D(int n1, int n2)
{
	const double F00= 0.0113;	const double F01= 0.0838;	const double F02= 0.0113;
	const double F10= 0.0838;	const double F11= 0.6193;	const double F12= 0.0838;
	const double F20= 0.0113;	const double F21= 0.0838;	const double F22= 0.0113;

	const double	d4	= 1/(F00+F01+F10+F11);
	const double	d6	= 1/(F00+F01+F10+F11+F20+F21);
	int		x,y;
	for(x=1; x<n2-1; ++x) {
		for(y=1; y<n1-1; ++y) {
			m_dTmp2D[y][x]= F00*m_dDiff2D[y-1][x-1] + F01*m_dDiff2D[y-1][x] + F02*m_dDiff2D[y-1][x+1] + 
							F10*m_dDiff2D[y][x-1]   + F11*m_dDiff2D[y][x]   + F12*m_dDiff2D[y][x+1] + 
							F20*m_dDiff2D[y+1][x-1] + F21*m_dDiff2D[y+1][x] + F22*m_dDiff2D[y+1][x+1];
			
			// column boundaries
			m_dTmp2D[y][0]	  = (F01*m_dDiff2D[y-1][0] + F02*m_dDiff2D[y-1][1] + 
								 F11*m_dDiff2D[y][0]   + F12*m_dDiff2D[y][1] + 
								 F21*m_dDiff2D[y+1][0] + F22*m_dDiff2D[y+1][1]) * d6;
			m_dTmp2D[y][n2-1] = (F00*m_dDiff2D[y-1][n2-2] + F01*m_dDiff2D[y-1][n2-1] + 
						 		 F10*m_dDiff2D[y][n2-2]   + F11*m_dDiff2D[y][n2-1] + 
								 F20*m_dDiff2D[y+1][n2-2] + F21*m_dDiff2D[y+1][n2-1]) * d6;
		}

		// row boundaries
		m_dTmp2D[0][x]	  = (F10*m_dDiff2D[0][x-1] + F11*m_dDiff2D[0][x] + F12*m_dDiff2D[0][x+1] + 
							 F20*m_dDiff2D[1][x-1] + F21*m_dDiff2D[1][x] + F22*m_dDiff2D[1][x+1]) * d6;
		m_dTmp2D[n1-1][x] = (F00*m_dDiff2D[n1-2][x-1] + F01*m_dDiff2D[n1-2][x] + F02*m_dDiff2D[n1-2][x+1] + 
							 F10*m_dDiff2D[n1-1][x-1] + F11*m_dDiff2D[n1-1][x] + F12*m_dDiff2D[n1-1][x+1]) * d6;
	}

	// corners
	m_dTmp2D[0][0]= (F11*m_dDiff2D[0][0] + F12*m_dDiff2D[0][1] +			
					 F21*m_dDiff2D[1][0] + F22*m_dDiff2D[1][1])*d4;						// left-bottom
	m_dTmp2D[0][n2-1] = (F10*m_dDiff2D[0][n2-2] + F11*m_dDiff2D[0][n2-1] + 
						 F20*m_dDiff2D[1][n2-2] + F21*m_dDiff2D[1][n2-1])*d4;			// right-bottom
	m_dTmp2D[n1-1][n2-1] = (F00*m_dDiff2D[n1-2][n2-2] + F01*m_dDiff2D[n1-2][n2-1] + 
							F10*m_dDiff2D[n1-1][n2-2] + F11*m_dDiff2D[n1-1][n2-1])*d4;	// right-top
	m_dTmp2D[n1-1][0] = (F01*m_dDiff2D[n1-2][0] + F02*m_dDiff2D[n1-2][1] + 
						 F11*m_dDiff2D[n1-1][0] + F12*m_dDiff2D[n1-1][1])*d4;			// left-top

	// copy result
    //memcpy(m_dDiff2D, m_dTmp2D, sizeof(double)*n1*n2);
	for(y=0; y<n1; ++y)	       for(x=0; x<n2; ++x)	            m_dDiff2D[y][x]	= m_dTmp2D[y][x];
}


//function that computes the distances from the vertex ind to all the neighbours
void descriptor_dists(vector <double *> &descriptor, int sigSize, vector<int> &neighbors, int ind, vector<double> &dists)
{
	vector <int>::iterator It;

 	dd_dist	dd(sigSize, sigSize);
	for (It = neighbors.begin(); It != neighbors.end(); ++It)
	{
		int neigbInd = *It;
		dists.push_back (dd.dd2D(descriptor[neigbInd],descriptor[ind])); 
    }
}

