#include "Colorization.h"
#include "calcSpinCurvC.h"


#include <vcg/complex/algorithms/update/normal.h>
#include <vcg/complex/algorithms/update/topology.h>
#include <vcg/complex/algorithms/update/curvature.h>
#include <vcg/complex/algorithms/stat.h>
#include <vcg/simplex/edge/base.h>
#include <vcg/simplex/face/topology.h>




vector <double *> calcSpinCurv (MyMesh *mesh, int spinImageSize)
{
	
	vector <double *> descriptor;

	// calc normals
	tri::UpdateNormals<MyMesh>::PerVertexNormalized(*mesh);

	// calc binSize = median (edgeLengths)

	vector<tri::UpdateTopology<MyMesh>::PEdge> edges;
	tri::UpdateTopology<MyMesh>::FillUniqueEdgeVector(*mesh, edges,  false);

	vector <double> edgesLength;
	for (unsigned int i= 0 ; i< edges.size() ; i++)
	{
		edgesLength.push_back(((edges[i].v[0] -> P()) - (edges[i].v[1] -> P())).Norm());
	}
	size_t n = edges.size() / 2; 
    nth_element(edgesLength.begin(), edgesLength.begin()+n, edgesLength.end()); 

	double binSize =  edgesLength[n];

	// Hovav: This returen different result, but very close....
	vcg::Distribution<float> H;
    tri::Stat<MyMesh>::ComputeFaceEdgeDistribution(*mesh,H);
    float medianEdgeLen = H.Percentile(0.5);

	// calc spin images
	tri::UpdateTopology<MyMesh>::FaceFace(*mesh);
	tri::UpdateTopology<MyMesh>::VertexFace(*mesh);
	tri::UpdateCurvature<MyMesh>::PrincipalDirections(*mesh);


	int vertexCount = mesh -> vn;

	double curv, alpha, beta;
	MyVertex *thisPt, *pt;
	Point3f *normal;
	Point3f temp;
	/*
	printf ("\n");

    for(int k=0; k<vertexCount; k++) {
		pt = &(mesh -> vert[k]);
		normal =&(pt->N());
        curv = (pt->K1()+pt->K2())/2;
		printf ("%9.3f",curv);
	}
	printf ("\n \n");

    for(int k=0; k<vertexCount; k++) {
		pt = &(mesh -> vert[k]);
		normal =&(pt->N());
        curv = (pt->K1()+pt->K2())/2;
		float x = (*normal)[0], y = (*normal)[1], z = (*normal)[2];
		printf ("%8.3f %8.3f %8.3f; ... \n",x, y, z);
	}
	printf ("\n \n");

	*/
    for(int k=0; k<vertexCount; k++) {
		if (k%100 == 0) printf ("K is %d\n",k);
		double *spinIm = (double *)malloc (sizeof(double)*spinImageSize*spinImageSize);
        for(int i=0; i<spinImageSize*spinImageSize; i++)
            spinIm[i] = 0;    

		pt = &(mesh -> vert[k]);
		normal =&(pt->N());
        curv = (pt->K1()+pt->K2())/2;
		        
        for(int j=0; j<vertexCount; j++) {
			thisPt = &(mesh -> vert[j]);
            
			temp = thisPt->P()- pt->P();

			beta = (*normal)*(temp);          
            beta = beta - spinImageSize*binSize/2;
            if (beta > 0 || beta < -spinImageSize*binSize)
                continue;

            int b = (int)floor(-beta/binSize);
			if (b >= spinImageSize || b < 0)
                continue;

			alpha = (temp^(*normal)).Norm();
            int a = (int)floor(alpha/binSize);

			if (a >= spinImageSize || a < 0 )
                continue;

            spinIm[a*spinImageSize + b] += ((thisPt->K1()+thisPt->K2())/2 - curv);
        }
		descriptor.push_back (spinIm);
    }
  
	return descriptor;
}

	
